import { Observable } from 'rxjs';

let NOTIN = {};
const continueWith = (mapper) => (source) => new Observable((observer) => {
  let latestValue = NOTIN;
  let subscription = source.subscribe({
    next(v) {
      observer.next(latestValue = v);
    },
    error(e) {
      observer.error(e);
    },
    complete() {
      if (latestValue === NOTIN) observer.complete();
      else subscription = mapper(latestValue).subscribe(observer);
    }
  });
  return () => {
    subscription.unsubscribe();
  };
});

export { continueWith };
//# sourceMappingURL=continue-with.mjs.map
